/*

Copyright (c) 2003, Braulio Jos Solano Rojas, OpenIsis Verein
All rights reserved.

Redistribution and use in source and binary forms, with or without modification, are
permitted provided that the following conditions are met:

*	Redistributions of source code must retain the above copyright notice, this list of
	conditions and the following disclaimer.
*	Redistributions in binary form must reproduce the above copyright notice, this list of
	conditions and the following disclaimer in the documentation and/or other materials
	provided with the distribution.
*	Neither the name of the "OpenIsis Verein" nor the names of its contributors may
	be used to endorse or promote products derived from this software without specific
	prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND
CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

*/

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

/* True globals, no need for thread safety */
static int le_isis_result, le_isis_dbid;

#include "php.h"
#include "php_ini.h"
#include "ext/standard/info.h"
#include "php_isis.h"
#include "openisis.h"

ZEND_DECLARE_MODULE_GLOBALS(isis)

/* True global resources */
static int le_isis_result, le_isis_dbid;

/* {{{ isis_functions[]
 *
 * Every user visible function must have an entry in isis_functions[].
 */
function_entry isis_functions[] = {
	ZEND_FE(isis_open,	NULL)
	ZEND_FE(isis_close,	NULL)
	ZEND_FE(isis_search,	NULL)
	ZEND_FE(isis_query,	NULL)
	ZEND_FE(isis_terms,	NULL)
	ZEND_FE(isis_fetch_array,	NULL)
	ZEND_FE(isis_fetch_flat_array,	NULL)
    ZEND_FE(isis_fetch_flat_array_from_mfn,	NULL)
	ZEND_FE(isis_num_rows,	NULL)
	ZEND_FE(isis_data_seek,	NULL)
	ZEND_FE(isis_mfn_data_seek,	NULL)
	ZEND_FE(isis_free_result,	NULL)
	ZEND_FE(isis_last_mfn,	NULL)
	{NULL, NULL, NULL}	/* Must be the last line in isis_functions[] */
};
/* }}} */

/* {{{ isis_module_entry
 */
zend_module_entry isis_module_entry = {
#if ZEND_MODULE_API_NO >= 20010901
	STANDARD_MODULE_HEADER,
#endif
	"isis",
	isis_functions,
	ZEND_MINIT(isis),
	ZEND_MSHUTDOWN(isis),
	ZEND_RINIT(isis),
	ZEND_RSHUTDOWN(isis),
	ZEND_MINFO(isis),
#if ZEND_MODULE_API_NO >= 20010901
	"1.0.3",				/* Version number of the extension */
#endif
	STANDARD_MODULE_PROPERTIES
};
/* }}} */

#ifdef COMPILE_DL_ISIS
ZEND_GET_MODULE(isis)
#endif

/* {{{ PHP_INI
 */
PHP_INI_BEGIN()
    STD_PHP_INI_ENTRY("isis.default_dir",		NULL,	PHP_INI_ALL,	OnUpdateString,		default_dir,		zend_isis_globals,	isis_globals)
    STD_PHP_INI_ENTRY("isis.default_database",	NULL,	PHP_INI_ALL,	OnUpdateString,		default_database,	zend_isis_globals,	isis_globals)
PHP_INI_END()
/* }}} */

/* {{{ php_isis_init_globals
 */
static void php_isis_init_globals(zend_isis_globals *isis_globals)
{
	isis_globals->default_dir = NULL;
	isis_globals->default_database = NULL;
}
/* }}} */

/* {{{ php_isis_rtrim
 */
static char *php_isis_rtrim(char *string)
{
	int    i;
	int    c;

	for (i = 0, c = 0; string[i] != '\0'; i++)
		if ((string[i] != ' ') && (string[i] != '\t' ) && (string[i] != '\n' ) && (string[i] != '\r' ) && (string[i] != '\v' ))
			c = i;

	for (i = 0; string[i] != '\0'; i++)
		if (c < i)
			string[i] = '\0';

    return (string);
}
/* }}} */

/* {{{ php_isis_ltrim
 */
static char *php_isis_ltrim(char *string)
{
	int    i;
	int    c;

	for (i = 0, c = 0; (string[i] == ' ') || (string[i] == '\t' ) || (string[i] == '\n' ) || (string[i] == '\r' ) || (string[i] == '\v' ); i++);
		c = i;

	for (i = 0; string[i] != '\0'; i++)
		if (i < (((signed) strlen(string)) - c))
			string[i] = string[i + c];
		else
			string[i] = '\0';

	return (string);
}
/* }}} */

/* {{{ php_isis_trim
 */
static char *php_isis_trim(char *string)
{
	return (php_isis_ltrim(php_isis_rtrim(string)));
}
/* }}} */

/* {{{ php_isis_subrow_array
 */
static zval *php_isis_subrow_array(OpenIsisRec *row, int i)
{
    OpenIsisRec *subrow;
    zval *z_array;
	zval *z_field_array;
	int j;
	char subfield_tag[3]="";

	if ((i-1>=0 && (row->field[i-1].tag == row->field[i].tag)) || (i+1<row->len && (row->field[i].tag == row->field[i+1].tag)))
	{
		MAKE_STD_ZVAL(z_field_array);

		if(array_init(z_field_array) != SUCCESS)
		{
			return NULL;
		}

		do
		{
			subrow = openIsisReadField(NULL, row->field+i);

			MAKE_STD_ZVAL(z_array);

			if(array_init(z_array) != SUCCESS)
			{
				return NULL;
			}

			for (j=0; j<subrow->len; j++)
			{
				sprintf(subfield_tag, "%c", (char)subrow->field[j].tag);
				if(add_assoc_stringl(z_array, subfield_tag,  subrow->field[j].val, (int)subrow->field[j].len, 1) == FAILURE)
				{
					return NULL;
				}
			}
			openIsisMFree(subrow);
			if(zend_hash_next_index_insert(HASH_OF(z_field_array), (void *)&z_array, sizeof(zval *), NULL) == FAILURE)
			{
				return NULL;
			}
			i++;
		}
		while ((i-1>=0 && (row->field[i-1].tag == row->field[i].tag)) || (i+1<row->len && (row->field[i].tag == row->field[i+1].tag)));

		return z_field_array;
	}
	else
	{
		subrow = openIsisReadField(NULL, row->field+i);

		MAKE_STD_ZVAL(z_array);

		if(array_init(z_array) != SUCCESS)
		{
			return NULL;
		}

		for (j=0; j<subrow->len; j++)
		{
			sprintf(subfield_tag, "%c", (char)subrow->field[j].tag);
			if(add_assoc_stringl(z_array, subfield_tag,  subrow->field[j].val, (int)subrow->field[j].len, 1) == FAILURE)
			{
				return NULL;
			}
		}
		openIsisMFree(subrow);
		return z_array;
	}
	return NULL;
}
/* }}} */


/* {{{ php_isis_fetch_array
 */
static zval *php_isis_fetch_array()
{
}
/* }}} */


/* {{{ proto resource isis_open(string dbname, string args)
   Return a dbid to manipulate an isis database */
ZEND_FUNCTION(isis_open)
{
	char *dbname = NULL;
	char *dbpath = "-dbpath";
	char *arg;
	char *args;
	char *argv[32];
	int  argc=0;
	int s_len;
	isis_dbid *dbid;
	char warning[256];

	if (ISIS_G(default_dir))
	{
		argv[0] = dbpath;
		argv[1] = ISIS_G(default_dir);
		argc+= 2;
	}

	switch(ZEND_NUM_ARGS())
	{
		case 0:
			if (ISIS_G(default_database))
			{
				dbname = ISIS_G(default_database);
			}
			else
			{
				zend_error(E_WARNING, "If not isis.default_database, at least one parameter must be provided");
				RETURN_FALSE;
			}
			break;
		case 1:
        	if (zend_parse_parameters(1 TSRMLS_CC, "s", &dbname, &s_len)==FAILURE)
			{
				RETURN_FALSE;
			}
			break;
		case 2:
        	if (zend_parse_parameters(2 TSRMLS_CC, "ss", &dbname, &s_len, &args, &s_len)==FAILURE)
			{
				RETURN_FALSE;
			}

			args = php_isis_trim(args);

			while (((arg = strtok(args, " \t\r\n\v")) != NULL) && (argc < 31))
			{
				argv[argc] = arg;
				++argc;
			}
			argv[argc] = NULL;

			break;
		default:
			WRONG_PARAM_COUNT;
			break;
	}

	dbname = php_isis_trim(dbname);
	dbid = emalloc(sizeof(isis_dbid));
	dbid->id = openIsisOpen(dbname, argv, argc);
	if ( 0 > dbid->id )
	{
		efree(dbid);
		sprintf(warning, "The database %s can not be opened!", dbname);
		zend_error(E_WARNING, warning);
		RETURN_FALSE;
	}
	dbid->dbname = estrdup(dbname);

    RETURN_RESOURCE(zend_list_insert(dbid, le_isis_dbid));
}
/* }}} */

/* {{{ proto boolean isis_close(resource isis_dbid)
   Return true or false on attempt to close an isis db */
ZEND_FUNCTION(isis_close)
{
    isis_dbid *dbid;
	zval *z_dbid;

	switch(ZEND_NUM_ARGS())
	{
		case 1:
        	if (zend_parse_parameters(1 TSRMLS_CC, "r", &z_dbid)==FAILURE)
			{
				RETURN_FALSE;
			}
			ZEND_FETCH_RESOURCE(dbid, isis_dbid *, &z_dbid, -1, "Isis dbid", le_isis_dbid);
			break;
		default:
			WRONG_PARAM_COUNT;
			break;
	}

	if (openIsisClose(dbid->id) < 0)
	{
		zend_error(E_WARNING, "I can't successfuly close the database!");
		RETURN_FALSE;
	}

	zend_list_delete(z_dbid->value.lval);

	RETURN_TRUE;
}
/* }}} */

/* {{{ proto resource isis_search(string term, resource isis_dbid)
   Return search for term over an isis database */
ZEND_FUNCTION(isis_search)
{
    char *term=NULL;
    long searchmode=OPENISIS_QRY_KEYAT;
	zval *z_dbid;
    isis_dbid *dbid;
	isis_result *result;
	int s_len;

	switch(ZEND_NUM_ARGS())
	{
		case 2:
        	if (zend_parse_parameters(2 TSRMLS_CC, "sr", &term, &s_len, &z_dbid)==FAILURE)
			{
				RETURN_FALSE;
			}
			ZEND_FETCH_RESOURCE(dbid, isis_dbid *, &z_dbid, -1, "Isis dbid", le_isis_dbid);
			break;
		default:
			WRONG_PARAM_COUNT;
			RETURN_FALSE;
	}

	result = emalloc(sizeof(isis_result));
	result->index=0;
    result->set.len = 0;
	openIsisQuery(&(result->set), dbid->id, term, searchmode, 0);

	if ( 0 >= result->set.len )
	{
		efree(result);
		RETURN_FALSE;
	}

	result->dbid = dbid->id;

    RETURN_RESOURCE(zend_list_insert(result, le_isis_result));
}
/* }}} */

/* {{{ proto resource isis_query(string querystring, resource isis_dbid)
   Return a set of records or empty on running a query like "water * plant" over isis */
ZEND_FUNCTION(isis_query)
{
    char *querystring=NULL;
    long searchmode=OPENISIS_QRY_SIMPLE;
	zval *z_dbid;
    isis_dbid *dbid;
	isis_result *result;
	int s_len;

	switch(ZEND_NUM_ARGS())
	{
		case 2:
        	if (zend_parse_parameters(2 TSRMLS_CC, "sr", &querystring, &s_len, &z_dbid)==FAILURE)
			{
				RETURN_FALSE;
			}
			ZEND_FETCH_RESOURCE(dbid, isis_dbid *, &z_dbid, -1, "Isis dbid", le_isis_dbid);
			break;
		default:
			WRONG_PARAM_COUNT;
			RETURN_FALSE;
	}

	result = emalloc(sizeof(isis_result));
	result->index=0;
    result->set.len = 0;
	openIsisQuery(&(result->set), dbid->id, querystring, searchmode, 0);

	if ( 0 >= result->set.len )
	{
		efree(result);
		RETURN_FALSE;
	}

	result->dbid = dbid->id;

    RETURN_RESOURCE(zend_list_insert(result, le_isis_result));
}
/* }}} */

/* {{{ proto array isis_terms(string term, resource isis_dbid)
   Return terms matching term (e.g. plant$) over an isis database */
ZEND_FUNCTION(isis_terms)
{
    char *term=NULL;
	zval *z_dbid;
    isis_dbid *dbid;
    int i;
	int s_len;
	union { OpenIsisRec set; char buf[10000]; } result;

	switch(ZEND_NUM_ARGS())
	{
		case 2:
        	if (zend_parse_parameters(2 TSRMLS_CC, "sr", &term, &s_len, &z_dbid)==FAILURE)
			{
				RETURN_FALSE;
			}
			ZEND_FETCH_RESOURCE(dbid, isis_dbid *, &z_dbid, -1, "Isis dbid", le_isis_dbid);
			break;
		default:
			WRONG_PARAM_COUNT;
			RETURN_FALSE;
	}

	if(array_init(return_value) != SUCCESS)
	{
		zend_error(E_WARNING, "I can't successfuly init the array!");
		RETURN_FALSE;
	}

	result.set.len   = 0;
	result.set.bytes = sizeof(result);
	while ( NULL != openIsisTerm(&(result.set), dbid->id, term) && result.set.len )
	{
		for (i=0; i<result.set.len; i++ )
		{
			if (add_next_index_stringl(return_value,  result.set.field[i].val, result.set.field[i].len, 1) == FAILURE)
			{
				zend_error(E_WARNING, "I can't successfuly complete the array!");
				RETURN_FALSE;
			}
		}
	}

}
/* }}} */

/* {{{ proto array isis_fetch_array(resource isis_result)
   Return an array with result information (mfn and tags) */
ZEND_FUNCTION(isis_fetch_array)
{
	zval *z_result;
	isis_result *result;
    isis_dbid dbid;
    OpenIsisRec *row;
    int i;
    zval *z_array;
	zval *z_field_array;

	switch(ZEND_NUM_ARGS())
	{
		case 1:
        	if (zend_parse_parameters(1 TSRMLS_CC, "r", &z_result)==FAILURE)
			{
				RETURN_FALSE;
			}
			ZEND_FETCH_RESOURCE(result, isis_result *, &z_result, -1, "Isis result", le_isis_result);
			break;
		default:
			WRONG_PARAM_COUNT;
			break;
	}

	if(array_init(return_value) != SUCCESS)
	{
		zend_error(E_WARNING, "I can't successfuly init the array!");
		RETURN_FALSE;
	}

	dbid.id = result->dbid;

	if (result->index < result->set.len)
	{
		row = openIsisReadRow(dbid.id, result->set.id[result->index]);
		if (row)
		{
			if(add_assoc_long(return_value, "mfn", row->rowid) == FAILURE)
			{
				zend_error(E_WARNING, "I can't successfuly complete the array!");
				RETURN_FALSE;
			}
			for (i=0; i<row->len; i++)
			{
				if (row->field[i].len && '^' == *row->field[i].val)
				{
					z_array = php_isis_subrow_array(row, i);
					if(!z_array || (zend_hash_index_update(HASH_OF(return_value), row->field[i-1].tag, (void *)&z_array, sizeof(zval *), NULL) == FAILURE))
					{
						zend_error(E_WARNING, "I can't successfuly complete the array!");
						RETURN_FALSE;
					}
				}
				else
				{
					if ((i-1>=0 && (row->field[i-1].tag == row->field[i].tag)) || (i+1<row->len && (row->field[i].tag == row->field[i+1].tag)))
					{
						MAKE_STD_ZVAL(z_field_array);

						if(array_init(z_field_array) != SUCCESS)
						{
							zend_error(E_WARNING, "I can't successfuly init the array!");
							RETURN_FALSE;
						}

						while ((i-1>=0 && (row->field[i-1].tag == row->field[i].tag)) || (i+1<row->len && (row->field[i].tag == row->field[i+1].tag)))
						{
							if(add_next_index_stringl(z_field_array, row->field[i].val, row->field[i].len, 1) == FAILURE)
							{
								zend_error(E_WARNING, "I can't successfuly complete the array!");
								RETURN_FALSE;
							}
							i++;
						}
						if(zend_hash_index_update(HASH_OF(return_value), row->field[i-1].tag, (void *)&z_field_array, sizeof(zval *), NULL) == FAILURE)
						{
							zend_error(E_WARNING, "I can't successfuly complete the array!");
							RETURN_FALSE;
						}
					}
					else
					{
						if(add_index_stringl(return_value, row->field[i].tag, row->field[i].val, row->field[i].len, 1) == FAILURE)
						{
							zend_error(E_WARNING, "I can't successfuly complete the array!");
							RETURN_FALSE;
						}
					}
				}
			}
			result->index++;
			openIsisMFree(row);
		}
	}
	else
	{
		RETURN_FALSE;
	}
}
/* }}} */

/* {{{ proto array isis_fetch_flat_array(resource isis_result)
   Return an array with result information (mfn and tags) */
ZEND_FUNCTION(isis_fetch_flat_array)
{
	zval *z_result;
	isis_result *result;
    isis_dbid dbid;
    OpenIsisRec *row;
    int i;
	zval *z_tag_value_array;

	switch(ZEND_NUM_ARGS())
	{
		case 1:
        	if (zend_parse_parameters(1 TSRMLS_CC, "r", &z_result)==FAILURE)
			{
				RETURN_FALSE;
			}
			ZEND_FETCH_RESOURCE(result, isis_result *, &z_result, -1, "Isis result", le_isis_result);
			break;
		default:
			WRONG_PARAM_COUNT;
			break;
	}

	if(array_init(return_value) != SUCCESS)
	{
		zend_error(E_WARNING, "I can't successfuly init the array!");
		RETURN_FALSE;
	}

	dbid.id = result->dbid;

	if (result->index < result->set.len)
	{
		row = openIsisReadRow(dbid.id, result->set.id[result->index]);
		if (row)
		{
			if(add_assoc_long(return_value, "mfn", row->rowid) == FAILURE)
			{
				zend_error(E_WARNING, "I can't successfuly complete the array!");
				RETURN_FALSE;
			}
			for (i=0; i<row->len; i++)
			{
				MAKE_STD_ZVAL(z_tag_value_array);

				if(array_init(z_tag_value_array) != SUCCESS)
				{
					zend_error(E_WARNING, "I can't successfuly init the array!");
					RETURN_FALSE;
				}

				if(add_index_long(z_tag_value_array, 0, row->field[i].tag) == FAILURE)
				{
					zend_error(E_WARNING, "I can't successfuly complete the array!");
					RETURN_FALSE;
				}

				if(add_next_index_stringl(z_tag_value_array, row->field[i].val, row->field[i].len, 1) == FAILURE)
				{
					zend_error(E_WARNING, "I can't successfuly complete the array!");
					RETURN_FALSE;
				}

				if(zend_hash_next_index_insert(HASH_OF(return_value), (void *)&z_tag_value_array, sizeof(zval *), NULL) == FAILURE)
				{
					zend_error(E_WARNING, "I can't successfuly complete the array!");
					RETURN_FALSE;
				}
			}
			result->index++;
			openIsisMFree(row);
		}
	}
	else
	{
		RETURN_FALSE;
	}
}
/* }}} */

/* {{{ proto array isis_fetch_flat_array_from_mfn(int mfn, resource isis_dbid)
   Return an array with result information (mfn and tags) */
ZEND_FUNCTION(isis_fetch_flat_array_from_mfn)
{
	zval *z_dbid;
    isis_dbid *dbid;
    OpenIsisRec *row;
    int i;
	int max_id;
	zval *z_tag_value_array;
	long mfn;

	switch(ZEND_NUM_ARGS())
	{
		case 2:
        	if (zend_parse_parameters(2 TSRMLS_CC, "lr", &mfn, &z_dbid)==FAILURE)
			{
				RETURN_FALSE;
			}
			ZEND_FETCH_RESOURCE(dbid, isis_dbid *, &z_dbid, -1, "Isis dbid", le_isis_dbid);
			break;
		default:
			WRONG_PARAM_COUNT;
			break;
	}

	if(array_init(return_value) != SUCCESS)
	{
		zend_error(E_WARNING, "I can't successfuly init the array!");
		RETURN_FALSE;
	}

	max_id = openIsisDMaxId(dbid->id);

	/* Checks that the number of the last record is safe */
	if(max_id <= 0)
	{
		zend_error(E_WARNING, "Impossible to determine how many records exists in database!");
		RETURN_FALSE;
	}

	/* Checks that the MFN is into a safe range */
	if (mfn <= max_id)
	{
		row = openIsisReadRow(dbid->id, mfn);
		if (row)
		{
			if(add_assoc_long(return_value, "mfn", row->rowid) == FAILURE)
			{
				zend_error(E_WARNING, "I can't successfuly complete the array!");
				RETURN_FALSE;
			}
			for (i=0; i<row->len; i++)
			{
				MAKE_STD_ZVAL(z_tag_value_array);

				if(array_init(z_tag_value_array) != SUCCESS)
				{
					zend_error(E_WARNING, "I can't successfuly init the array!");
					RETURN_FALSE;
				}

				if(add_index_long(z_tag_value_array, 0, row->field[i].tag) == FAILURE)
				{
					zend_error(E_WARNING, "I can't successfuly complete the array!");
					RETURN_FALSE;
				}

				if(add_next_index_stringl(z_tag_value_array, row->field[i].val, row->field[i].len, 1) == FAILURE)
				{
					zend_error(E_WARNING, "I can't successfuly complete the array!");
					RETURN_FALSE;
				}

				if(zend_hash_next_index_insert(HASH_OF(return_value), (void *)&z_tag_value_array, sizeof(zval *), NULL) == FAILURE)
				{
					zend_error(E_WARNING, "I can't successfuly complete the array!");
					RETURN_FALSE;
				}
			}
			openIsisMFree(row);
		}
	}
	else
	{
		zend_error(E_WARNING, "The mfn number is out of range!");
		RETURN_FALSE;
	}
}
/* }}} */

/* {{{ proto int isis_num_rows(resource isis_result)
   Return the number of rows of a result */
ZEND_FUNCTION(isis_num_rows)
{
	zval *z_result;
	isis_result *result;

	switch(ZEND_NUM_ARGS())
	{
		case 1:
        	if (zend_parse_parameters(1 TSRMLS_CC, "r", &z_result)==FAILURE)
			{
				RETURN_FALSE;
			}
			ZEND_FETCH_RESOURCE(result, isis_result *, &z_result, -1, "Isis result", le_isis_result);
			break;
		default:
			WRONG_PARAM_COUNT;
			break;
	}

	RETURN_LONG(result->set.len);
}
/* }}} */

/* {{{ proto boolean isis_data_seek(resource isis_result, int row_number)
   Move internal result pointer to row */
ZEND_FUNCTION(isis_data_seek)
{
	zval *z_result;
	isis_result *result;
	long rownumber;

	switch(ZEND_NUM_ARGS())
	{
		case 2:
        	if (zend_parse_parameters(2 TSRMLS_CC, "rl", &z_result, &rownumber)==FAILURE)
			{
				RETURN_FALSE;
			}
			ZEND_FETCH_RESOURCE(result, isis_result *, &z_result, -1, "Isis result", le_isis_result);
			break;
		default:
			WRONG_PARAM_COUNT;
			break;
	}

	if (rownumber >= result->set.len)
	{
		RETURN_FALSE;
	}

	result->index = rownumber;

	RETURN_TRUE;
}
/* }}} */

/* {{{ proto boolean isis_mfn_data_seek(resource isis_result, int mfn_number)
   Move internal result pointer to mfn */
ZEND_FUNCTION(isis_mfn_data_seek)
{
	zval *z_result;
	isis_result *result;
	long rownumber;
	long mfnnumber;

	switch(ZEND_NUM_ARGS())
	{
		case 2:
        	if (zend_parse_parameters(2 TSRMLS_CC, "rl", &z_result, &mfnnumber)==FAILURE)
			{
				RETURN_FALSE;
			}
			ZEND_FETCH_RESOURCE(result, isis_result *, &z_result, -1, "Isis result", le_isis_result);
			break;
		default:
			WRONG_PARAM_COUNT;
			break;
	}

	for (rownumber=0; rownumber<result->set.len; ++rownumber)
		if (result->set.id[rownumber] == mfnnumber) break;

	if (rownumber >= result->set.len)
	{
		RETURN_FALSE;
	}

	result->index = rownumber;

	RETURN_TRUE;
}
/* }}} */

/* {{{ proto boolean isis_free_result(resource isis_result)
   Return TRUE or FALSE */
ZEND_FUNCTION(isis_free_result)
{
	zval *z_result;
	isis_result *result;

	switch(ZEND_NUM_ARGS())
	{
		case 1:
        	if (zend_parse_parameters(1 TSRMLS_CC, "r", &z_result)==FAILURE)
			{
				RETURN_FALSE;
			}
			ZEND_FETCH_RESOURCE(result, isis_result *, &z_result, -1, "Isis result", le_isis_result);
			break;
		default:
			WRONG_PARAM_COUNT;
			break;
	}

	zend_list_delete(z_result->value.lval);

	RETURN_TRUE;
}
/* }}} */


/* {{{ proto boolean isis_last_mfn(resource isis_dbid)
   Return the last MFN in database or FALSE */
ZEND_FUNCTION(isis_last_mfn)
{
	zval *z_dbid;
    isis_dbid *dbid;
	int max_id;

	switch(ZEND_NUM_ARGS())
	{
		case 1:
        	if (zend_parse_parameters(1 TSRMLS_CC, "r", &z_dbid)==FAILURE)
			{
				RETURN_FALSE;
			}
			ZEND_FETCH_RESOURCE(dbid, isis_dbid *, &z_dbid, -1, "Isis dbid", le_isis_dbid);
			break;
		default:
			WRONG_PARAM_COUNT;
			break;
	}

	max_id = openIsisDMaxId(dbid->id);

	/* Checks that the number of the last record is safe */
	if(max_id <= 0)
	{
		zend_error(E_WARNING, "Impossible to determine how many records exists in database!");
		RETURN_FALSE;
	}

	RETURN_LONG(max_id);
}
/* }}} */


/* {{{ proto boolean isis_write(array record, resource isis_dbid)
   Return TRUE or FALSE */
ZEND_FUNCTION(isis_write)
{
	char *ftok="ftok";
	char *sem_get="sem_get";
	char *sem_acquire="sem_acquire";
	char *sem_release="sem_release";
	char *sem_remove="sem_remove";
    isis_dbid *dbid;
	zval *z_function_name;
	zval *z_params[2];
	zval *z_param1;
	zval *z_param2;
	zval *z_retval;
	zval *z_dbid;
	zval *z_array;

	switch(ZEND_NUM_ARGS())
	{
		case 2:
        	if (zend_parse_parameters(2 TSRMLS_CC, "ar", &z_array, &z_dbid)==FAILURE)
			{
				RETURN_FALSE;
			}
			ZEND_FETCH_RESOURCE(dbid, isis_dbid *, &z_dbid, -1, "Isis dbid", le_isis_dbid);
			break;
		default:
			WRONG_PARAM_COUNT;
			RETURN_FALSE;
	}

#ifndef WIN32
#if HAVE_SYSVSEM

	MAKE_STD_ZVAL(z_param1);
	ZVAL_STRING(z_param1, dbid->dbname, 1);

	MAKE_STD_ZVAL(z_param2);
	ZVAL_STRING(z_param2, "OpenIsis", 1);

	MAKE_STD_ZVAL(z_function_name);
	ZVAL_STRING(z_function_name, ftok, 1);

	CLS_FETCH();

	if(call_user_function_ex(CG(function_table), NULL, z_function_name, &z_retval, 2, &z_params, 0, NULL TSRMLS_CC) != SUCCESS)
    {
        zend_error(E_WARNING, "Call to ftok failed");
		RETURN_FALSE;
    }


#endif /* HAVE_SYSVSEM */
#endif /* WIN32 */

	RETURN_FALSE;
}
/* }}} */

/* {{{ isis_result_destructor
 */
static void isis_result_destructor(isis_result *result)
{
	efree(result);
}
/* }}} */

/* {{{ isis_dbid_destructor
 */
static void isis_dbid_destructor(isis_dbid *dbid)
{
	efree(dbid);
}
/* }}} */

/* {{{ ZEND_MINIT_FUNCTION
 */
ZEND_MINIT_FUNCTION(isis)
{
	ZEND_INIT_MODULE_GLOBALS(isis, php_isis_init_globals, NULL);
	REGISTER_INI_ENTRIES();

	le_isis_result = register_list_destructors(isis_result_destructor, NULL);
	le_isis_dbid = register_list_destructors(isis_dbid_destructor, NULL);

	return(SUCCESS);
}
/* }}} */

/* {{{ ZEND_MSHUTDOWN_FUNCTION
 */
ZEND_MSHUTDOWN_FUNCTION(isis)
{
	UNREGISTER_INI_ENTRIES();
	return SUCCESS;
}
/* }}} */

/* Remove if there's nothing to do at request start */
/* {{{ ZEND_RINIT_FUNCTION
 */
ZEND_RINIT_FUNCTION(isis)
{
	return SUCCESS;
}
/* }}} */

/* Remove if there's nothing to do at request end */
/* {{{ ZEND_RSHUTDOWN_FUNCTION
 */
ZEND_RSHUTDOWN_FUNCTION(isis)
{
	return SUCCESS;
}
/* }}} */

/* {{{ ZEND_MINFO_FUNCTION
 */
ZEND_MINFO_FUNCTION(isis)
{
	php_info_print_table_start();
	php_info_print_table_row(2, "OpenIsis library support (http://www.openisis.org)", "enabled");
	php_info_print_table_end();

	DISPLAY_INI_ENTRIES();
}
/* }}} */


/*
 * Local variables:
 * tab-width: 4
 * c-basic-offset: 4
 * End:
 * vim600: sw=4 ts=4 tw=78 fdm=marker
 * vim<600: sw=4 ts=4 tw=78
 */
