/*

Copyright (c) 2003, Braulio Jos Solano Rojas, OpenIsis Verein
All rights reserved.

Redistribution and use in source and binary forms, with or without modification, are
permitted provided that the following conditions are met:

*	Redistributions of source code must retain the above copyright notice, this list of
	conditions and the following disclaimer.
*	Redistributions in binary form must reproduce the above copyright notice, this list of
	conditions and the following disclaimer in the documentation and/or other materials
	provided with the distribution.
*	Neither the name of the "OpenIsis Verein" nor the names of its contributors may
	be used to endorse or promote products derived from this software without specific
	prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND
CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

*/

#ifndef PHP_ISIS_H
#define PHP_ISIS_H

extern zend_module_entry isis_module_entry;
#define phpext_isis_ptr &isis_module_entry

#ifdef PHP_WIN32
#define PHP_ISIS_API __declspec(dllexport)
#else
#define PHP_ISIS_API
#endif

#ifdef ZTS
#include "TSRM.h"
#endif

#include "openisis.h"

/* Resources types */
typedef struct
{
	long index;
	OpenIsisSet set;
	int dbid;
}
isis_result;

typedef struct
{
	int id;
	char *dbname;
}
isis_dbid;


ZEND_MINIT_FUNCTION(isis);
ZEND_MSHUTDOWN_FUNCTION(isis);
ZEND_RINIT_FUNCTION(isis);
ZEND_RSHUTDOWN_FUNCTION(isis);
ZEND_MINFO_FUNCTION(isis);

/* Functions */
ZEND_FUNCTION(isis_open);
ZEND_FUNCTION(isis_close);
ZEND_FUNCTION(isis_search);
ZEND_FUNCTION(isis_query);
ZEND_FUNCTION(isis_terms);
ZEND_FUNCTION(isis_fetch_array);
ZEND_FUNCTION(isis_fetch_flat_array);
ZEND_FUNCTION(isis_fetch_flat_array_from_mfn);
ZEND_FUNCTION(isis_num_rows);
ZEND_FUNCTION(isis_data_seek);
ZEND_FUNCTION(isis_mfn_data_seek);
ZEND_FUNCTION(isis_free_result);
ZEND_FUNCTION(isis_write);
ZEND_FUNCTION(isis_last_mfn);
static void isis_result_destructor(isis_result *result);
static void isis_dbid_destructor(isis_dbid *dbid);

ZEND_BEGIN_MODULE_GLOBALS(isis)
	char *default_dir;
	char *default_database;
ZEND_END_MODULE_GLOBALS(isis)

/* In every utility function you add that needs to use variables
   in php_isis_globals, call TSRM_FETCH(); after declaring other
   variables used by that function, or better yet, pass in TSRMG_CC
   after the last function argument and declare your utility function
   with TSRMG_DC after the last declared argument.  Always refer to
   the globals in your function as ISIS_G(variable).  You are
   encouraged to rename these macros something shorter, see
   examples in any other php module directory.
*/

#ifdef ZTS
#define ISIS_G(v) TSRMG(isis_globals_id, zend_isis_globals *, v)
#else
#define ISIS_G(v) (isis_globals.v)
#endif

#endif	/* PHP_ISIS_H */


/*
 * Local variables:
 * tab-width: 4
 * c-basic-offset: 4
 * indent-tabs-mode: t
 * End:
 */
